#include "StdAfx.h"
#include "ProfileDlgController.h"
#include "ProfileWnd.h"
#include "NeMessengerDlg.h"
#include "main_frame.hpp"

#define PROFILE_DIALOG_WIDTH 205  // refer to UX document
#define PROFILE_DIALOG_HEIGHT 303 // refer to UX document
#define HANATOUR_OFFSET	76

CProfileDlgController::CProfileDlgController(void)
{
	m_dlgVector.clear();
	m_rcNewDlgRect.SetRectEmpty();

	for (int i = 0; i < MAX_PROFILE_DLG_NUM; i++)
		m_bDlgCreated[i] = false;
}

CProfileDlgController::~CProfileDlgController(void)
{
	for (PROFILE_DLG_ITER iter = m_dlgVector.begin(); iter != m_dlgVector.end(); iter++)
	{
		delete *iter;
		*iter = NULL;
	}
	m_dlgVector.clear();
}

BOOL CProfileDlgController::CreateProfileDlg(CString strUserID, RECT rcAround, HWND hWndParent)
{
	if(IsWindow(hWndParent) == FALSE)
		return FALSE;

	CloseProfileDlg();
	int xOffset = 5;
	int yOffset = 1;

	POINT pt = { 0 };
	GetCursorPos(&pt);

	RECT rcMonitor = { 0 };
	MONITORINFO oMonitor = {0};
	oMonitor.cbSize = sizeof(oMonitor);
	GetMonitorInfo(MonitorFromPoint(pt,MONITOR_DEFAULTTONEAREST),&oMonitor);
	rcMonitor = oMonitor.rcWork;

	pt.x = rcAround.left - PROFILE_DIALOG_WIDTH - xOffset;
	pt.y = rcAround.top + yOffset;

	if (pt.x < rcMonitor.left)
	{
		pt.x = rcAround.right + xOffset;
	}

	if (pt.y + PROFILE_DIALOG_HEIGHT > rcMonitor.bottom)
	{
		pt.y = rcAround.bottom - PROFILE_DIALOG_HEIGHT - yOffset;
		if (g_stNtalkInfo.strCorp != _T("hanatour"))
		{
			pt.y += HANATOUR_OFFSET;
		}
	}

	CRect rcWnd(pt.x, pt.y, pt.x + PROFILE_DIALOG_WIDTH, pt.y + PROFILE_DIALOG_HEIGHT);
	if (g_stNtalkInfo.strCorp != _T("hanatour"))
	{
		rcWnd.bottom -= HANATOUR_OFFSET;
	}

	if (g_pProfileWnd == NULL || !IsWindow(g_pProfileWnd->GetHWND()))
	{
		g_pProfileWnd = new CProfileWnd();
		g_pProfileWnd->SetParentHwnd(hWndParent);
		g_pProfileWnd->Create(hWndParent, _T("ProfileWindow"), WS_POPUP, WS_EX_TOOLWINDOW, rcWnd);
	}
	g_pProfileWnd->SetClientRect(rcWnd);
	g_pProfileWnd->SetRequestID(strUserID);
	if(IsWindow(hWndParent) == FALSE)
	{
		return FALSE;
	}

	return TRUE;
}

void CProfileDlgController::CloseProfileDlg()
{
	if (g_pProfileWnd != NULL && IsWindow(g_pProfileWnd->GetHWND()))
	{
		DestroyWindow(g_pProfileWnd->GetHWND());
	}
}
//  â  ġ Ѵ.
CRect CProfileDlgController::GetNewDlgRect()
{
	if (m_rcNewDlgRect.IsRectNull())	//   â ó   ġ
	{
		m_rcNewDlgRect.left = GetSystemMetrics(SM_CXSCREEN) / 2;
		m_rcNewDlgRect.top = GetSystemMetrics(SM_CYSCREEN) / 2 - PROFILE_DLG_HEIGHT;
		if (m_rcNewDlgRect.top <= 0)
			m_rcNewDlgRect.top = 50;

		m_rcNewDlgRect.right = m_rcNewDlgRect.left + PROFILE_DLG_WIDTH;
		m_rcNewDlgRect.bottom = m_rcNewDlgRect.top + PROFILE_DLG_HEIGHT;
	}
	else
	{
		int nMargin = 50;	//  Ƿ .  ̵忡  ؾ   .
		int nOffset = GetSystemMetrics(SM_CYCAPTION);

		if ((m_rcNewDlgRect.right + nOffset + nMargin > GetSystemMetrics(SM_CXSCREEN)) || 
			(m_rcNewDlgRect.bottom + nOffset + nMargin > GetSystemMetrics(SM_CYSCREEN)))
		{
			m_rcNewDlgRect.MoveToXY(nOffset, nOffset);
		}
		else
		{
			m_rcNewDlgRect.OffsetRect(nOffset, nOffset);
		}
	}

	return m_rcNewDlgRect;
}

int CProfileDlgController::GetCount()
{
	return m_dlgVector.size();
}

void CProfileDlgController::Add(CProfileDlg* pProfileDlg)
{
	m_dlgVector.push_back(pProfileDlg);
}

void CProfileDlgController::CloseAllDlg()
{
	for (PROFILE_DLG_ITER iter = m_dlgVector.begin(); iter != m_dlgVector.end(); iter++)
	{
		CProfileDlg *pDlg = *iter;

		if (pDlg->GetSafeHwnd())
			pDlg->DestroyWindow();
	}
	m_dlgVector.clear();

	CSingleLock lock(&m_csProfileDlgID);
	lock.Lock();

	for (int i = 0; i < MAX_PROFILE_DLG_NUM; i++)
		m_bDlgCreated[i] = false;

	lock.Unlock();
}

void CProfileDlgController::CloseDlg(CString strID, int nDlgID)
{
	RemoveDlgID(nDlgID);

	for (PROFILE_DLG_ITER iter = m_dlgVector.begin(); iter != m_dlgVector.end(); iter++)
	{
		CProfileDlg *pDlg = *iter;

		if (pDlg->GetID().CompareNoCase(strID) == 0)
		{
			m_dlgVector.erase(iter);

			if (pDlg->GetSafeHwnd())
				pDlg->DestroyWindow();

			break;
		}
	}
}

void CProfileDlgController::SetReadOnly(BOOL bStatus)
{
	for (PROFILE_DLG_ITER iter = m_dlgVector.begin(); iter != m_dlgVector.end(); iter++)
	{
		CProfileDlg *pDlg = *iter;

		if (pDlg->GetSafeHwnd())
			pDlg->SetReadOnly(bStatus);
	}
}

CProfileDlg* CProfileDlgController::GetProfileDlg(CString strID)
{
	for (PROFILE_DLG_ITER iter = m_dlgVector.begin(); iter != m_dlgVector.end(); iter++)
	{
		CProfileDlg *pDlg = *iter;

		if (pDlg->GetID().CompareNoCase(strID) == 0)
			return pDlg;
	}

	return NULL;
}

int CProfileDlgController::CreateDlgID()
{
	CSingleLock lock(&m_csProfileDlgID);
	lock.Lock();

	for (int i = 0; i < MAX_PROFILE_DLG_NUM; i++)
	{
		if (m_bDlgCreated[i] == false)
		{
			m_bDlgCreated[i] = true;
			lock.Unlock();

			return i;
		}
	}

	lock.Unlock();

	return -1;
}

void CProfileDlgController::RemoveDlgID(int nDlgID)
{
	CSingleLock lock(&m_csProfileDlgID);
	lock.Lock();

	m_bDlgCreated[nDlgID] = false;

	lock.Unlock();
}
